@@ -40,7 +40,7 @@ import msgparsing
 from diff import seddiff
 
 
-VERSION = '1.16'
+VERSION = '1.17'
 
 
 class IrcDisconnectError(Exception): ...
@@ -946,16 +946,16 @@ def main() -> None:
     # Parameters are dealt with
 
     async def irc_listener() -> None:
+        loop = asyncio.get_running_loop()
+
         serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         serversocket.bind((ip, port))
         serversocket.listen(1)
+        serversocket.setblocking(False)
 
-        s, _ = serversocket.accept()
+        s, _ = await loop.sock_accept(serversocket)
         serversocket.close()
-        asyncio.get_running_loop().add_signal_handler(signal.SIGHUP, term_f)
-        asyncio.get_running_loop().add_signal_handler(signal.SIGTERM, term_f)
-        asyncio.get_running_loop().add_signal_handler(signal.SIGINT, term_f)
         reader, writer = await asyncio.open_connection(sock=s)
 
         previous_status = None
@@ -997,14 +997,20 @@ def main() -> None:
             from_irc_task.cancel()
             to_irc_task.cancel()
 
-    while True:
-        signal.signal(signal.SIGHUP, term_f)
-        signal.signal(signal.SIGTERM, term_f)
-        signal.signal(signal.SIGINT, term_f)
-        try:
-            asyncio.run(irc_listener())
-        except IrcDisconnectError:
-            log('IRC disconnected')
+    async def restart_listener_loop():
+        loop = asyncio.get_running_loop()
+
+        loop.add_signal_handler(signal.SIGHUP, term_f)
+        loop.add_signal_handler(signal.SIGTERM, term_f)
+        loop.add_signal_handler(signal.SIGINT, term_f)
+
+        while True:
+            try:
+                await irc_listener()
+            except IrcDisconnectError:
+                log('IRC disconnected')
+
+    asyncio.run(restart_listener_loop())
 
 
 async def from_irc(reader, ircclient: Client):
@@ -1015,6 +1021,7 @@ async def from_irc(reader, ircclient: Client):
             raise IrcDisconnectError()
         await ircclient.command(cmd.strip())
 
+
 async def to_irc(sl_client: slack.Slack, ircclient: Client):
     while True:
         ev = await sl_client.event()
@@ -1022,9 +1029,11 @@ async def to_irc(sl_client: slack.Slack, ircclient: Client):
             log(ev)
             await ircclient.slack_event(ev)
 
+
 def term_f(*args):
     sys.exit(0)
 
+
 if __name__ == '__main__':
     try:
         main()
